@@ -4,6 +4,7 @@ from scipy.integrate import cumtrapz, odeint
 
 from scipy.interpolate import interp1d
 from scipy.stats import norm
+from scipy import signal
 
 from ..calc.cfc import CFC
 from ..calc.risk_function_util import DCDF, RibCriteria
@@ -227,6 +228,7 @@ class StandardFunction:
             limit_acc_x:float,
             limit_acc_y:float,
             limit_acc_z:float,
+            filter_option:str,
             units):
         """
 
@@ -237,39 +239,118 @@ class StandardFunction:
         :param limit_acceleration_x:
         :param lim_acceleration_y:
         :param lim_acceleration_z:
+        :param filter_option:
+        :param units:
         :return:
         """
         g = 9.81
 
         # average over time
         N = np.where(time_.flatten() / units.second() < 0.05)[0][-1]  # 50 ms
-        import matplotlib.pyplot as plt
 
-        plt.plot(acc_x)
-        plt.show()
         acc_x /= units.meter() / (units.second() ** 2) * g # acc_x in [m/s^2]
         acc_y /= units.meter() / (units.second() ** 2) * g # acc_y in [m/s^2]
         acc_z /= units.meter() / (units.second() ** 2) * g # acc_z in [m/s^2]
-        acceleration_x_average = np.convolve(acc_x.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
-        acceleration_y_average = np.convolve(acc_y.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
-        acceleration_z_average = np.convolve(acc_z.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
 
-        from scipy import signal
-        sos = signal.butter(2, 13, 'lowpass', fs=1000, output='sos')
-        acceleration_x_butter = signal.sosfilt(sos, acc_x.flatten()).reshape(-1, 1)
-        acceleration_y_butter = signal.sosfilt(sos, acc_y.flatten()).reshape(-1, 1)
-        acceleration_z_butter = signal.sosfilt(sos, acc_z.flatten()).reshape(-1, 1)
-        #acceleration_y_butter = np.convolve(acc_y.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
-        #acceleration_z_butter = np.convolve(acc_z.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
+        assert (filter_option in ["butter", "mean"])
 
-        plt.plot(acc_x)
-        plt.plot(acceleration_x_average)
-        plt.plot(acceleration_x_butter)
-        plt.show()
+        if filter_option == "mean":
+            acceleration_x = np.convolve(acc_x.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
+            acceleration_y = np.convolve(acc_y.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
+            acceleration_z = np.convolve(acc_z.flatten(), np.ones(N) / N, mode='same').reshape(-1, 1)
+        else:
+            sos = signal.butter(4, 13, 'lowpass', fs=1000, output='sos')
+            acceleration_x = signal.sosfilt(sos, acc_x.flatten()).reshape(-1, 1)
+            acceleration_y = signal.sosfilt(sos, acc_y.flatten()).reshape(-1, 1)
+            acceleration_z = signal.sosfilt(sos, acc_z.flatten()).reshape(-1, 1)
+
+        return np.sqrt((acceleration_x/limit_acc_x)**2
+                       + (acceleration_y/limit_acc_y)**2
+                        + (acceleration_z/limit_acc_z)**2)
+
+    @staticmethod
+    def ASI_from_velocity(time_:np.ndarray, vel_x:np.ndarray,
+            vel_y: np.ndarray, vel_z: np.ndarray,
+            limit_acc_x:float,
+            limit_acc_y:float,
+            limit_acc_z:float,
+            filter_option:str,
+            units):
+        """
+
+        :param time
+        :param acceleration_x:
+        :param acceleration_y:
+        :param acceleration_z:
+        :param limit_acceleration_x:
+        :param lim_acceleration_y:
+        :param lim_acceleration_z:
+        :param units:
+        :return:
+        """
+
+        acc_x = np.gradient(vel_x, axis=0)
+        acc_y = np.gradient(vel_y, axis=0)
+        acc_z = np.gradient(vel_z, axis=0)
+        return StandardFunction.ASI(time_, acc_x, acc_y, acc_z, limit_acc_x, limit_acc_y, limit_acc_z,
+                                    filter_option, units)
+
+    @staticmethod
+    def THIV(time_: np.ndarray, acc_x: np.ndarray,
+            acc_y: np.ndarray, rvel_z: np.ndarray,
+            Dx: float,
+            Dy: float,
+            units):
+        """
+
+        :param time
+        :param acc_x:
+        :param acc_y:
+        :param rotational_velocity_z:
+        :param Dx:
+        :param Dy:
+        :param units:
+        :return:
+        """
+        time_si = time_ / units.second()
+        time_si = time_si.flatten()
+
+        acc_x = np.gradient(acc_x, time_si, axis=0)
+        acc_y = np.gradient(acc_y, time_si, axis=0)
+
+        rvel_z *= units.second()
+        rvel_z = rvel_z.flatten()
+
+        acc_x = acc_x.flatten()
+        acc_y = acc_y.flatten()
+
+        rdisp_z = np.append(0, np.array([cumtrapz(rvel_z, x=time_si, axis=-1)]))
+
+        acc_x_floor = np.cos(rdisp_z) * acc_x - np.sin(rdisp_z) * acc_y
+        acc_y_floor = np.sin(rdisp_z) * acc_x + np.cos(rdisp_z) * acc_y
+
+        vel_x_floor = np.append(0, np.array([cumtrapz(acc_x_floor, x=time_si, axis=-1)]))
+        vel_y_floor = np.append(0, np.array([cumtrapz(acc_y_floor, x=time_si, axis=-1)]))
+
+        dist_x_floor = np.append(0, np.array([cumtrapz(vel_x_floor, x=time_si, axis=-1)]))
+        dist_y_floor = np.append(0, np.array([cumtrapz(vel_y_floor, x=time_si, axis=-1)]))
+
+        x_0 = 0
+        y_0 = 0
+
+        dist_head_x = ((x_0 - dist_x_floor) * np.cos(rdisp_z)) + ((y_0-dist_y_floor) * np.sin(rdisp_z))
+        dist_head_y = -((x_0 - dist_x_floor) * np.sin(rdisp_z)) + (y_0-dist_y_floor) * np.cos(rdisp_z)
+
+        # v_head
+        # cosψ −Y & c sinψ + yb(t) ψ &
+        vel_x_head = -vel_x_floor*np.cos(rdisp_z) - vel_y_floor*np.sin(rdisp_z) + dist_head_y * rvel_z
+        vel_y_head = vel_x_floor*np.sin(rdisp_z) - vel_y_floor*np.cos(rdisp_z) - dist_head_x * rvel_z
+
+        index_flight_time = np.min(np.where(np.logical_or(np.round(dist_head_x, 1) == Dx + x_0,
+                                                         (np.round(dist_head_y, 1) == Dy),
+                                                         (np.round(dist_head_y, 1) == -Dy))))
+        return (np.sqrt(vel_x_head**2 + vel_y_head**2) * 3.6)[:index_flight_time]
 
-        return np.sqrt((acceleration_x_butter/limit_acc_x)**2
-                       + (acceleration_y_butter/limit_acc_y)**2
-                        + (acceleration_z_butter/limit_acc_z)**2)
 
     @staticmethod
     def ccdf_normal_dist(x: float, mean: float, standard_deviation: float, units):
