@@ -4,6 +4,8 @@ import numpy as np
 
 from ..utils.constants import LOGConstants, TestConstants, JsonConstants
 import copy
+import lasso
+
 
 class BinoutData(object):
 
@@ -321,6 +323,8 @@ class BinoutData(object):
             return_function = self._check_id_in_binout("_abstat")
         elif self._name == "abstat_cpm":
             return_function = self._check_id_in_binout("_abstat_cpm")
+        elif self._name == "sleout":
+            return_function = self._check_id_in_binout("_sleout")
         if not return_function[0]:
             self._logger.emit(LOGConstants.WARNING[0], "Not valid ID: " + return_function[1] +
                               ". Plese check your def file!")
@@ -351,15 +355,15 @@ class BinoutData(object):
 
         for sec in dyn_def_key:
             temp_list = list(
-                map(functools.partial(BinoutData.map_help_function, binout_name, ids_list), dyn_def_key[sec]))
+                map(functools.partial(self._map_help_function, binout_name, ids_list), dyn_def_key[sec]))
 
             if len(temp_list) == 0:
                 return False, sec
             dyn_def_key[sec] = [tmp for tmp in temp_list if tmp != None].copy()
         return True, "temp"
 
-    @staticmethod
-    def map_help_function(binout_name, ids_list, id_):
+
+    def _map_help_function(self, binout_name, ids_list, id_):
         """
 
         :param binout_name:
@@ -367,15 +371,16 @@ class BinoutData(object):
         :param id_:
         :return:
         """
-        if binout_name == TestConstants.CONTACT:
+        if binout_name == TestConstants.CONTACT and isinstance(self._binout, lasso.dyna.Binout):
             master_slave = id_[-1]
             id_for_loop = int(id_[:-1])
-
             if id_for_loop in ids_list:
                 return str(id_for_loop) + master_slave
         elif id_ in ids_list:
             return id_
 
+        return None
+
     def _check_negative_intersection(self, elements):
         """
         :param: elements
