@@ -87,6 +87,7 @@ class DynasaurDefinitions:
         self._energy_part = {}
         self._energy_global = {}
         self._abstat = {}
+        self._sleout = {}
         self._abstat_cpm = {}
         self._riskfunction = {DefFileConstants.FUNC_CRITERIA: {},
                               DefFileConstants.FUNC_DATA_VISUALIZATION: {}}
@@ -122,6 +123,9 @@ class DynasaurDefinitions:
         """
         return self._units
 
+    def get_criteria_ids(self):
+        return [key for key in self._criteria.keys()]
+
     def get_criteria(self, key):
         """
         get criteria
@@ -135,6 +139,34 @@ class DynasaurDefinitions:
             return
         return self._criteria[key]
 
+    def get_data_vis_calc_commands(self):
+        """
+
+        :return:
+        """
+
+        ls = [key  for key in self._data_vis.keys()]
+        ret = []
+        for key in ls:
+            d = {"visualization": key}
+            if "label" in self._data_vis[key]["x"]:
+                x_label = self._data_vis[key]["x"]["label"]
+                d["x_label"] = x_label
+            if "label" in self._data_vis[key]["y"]:
+                y_label = self._data_vis[key]["y"]["label"]
+                d["y_label"] = y_label
+
+            ret.append(d)
+        return ret
+
+    def get_criteria_calc_commands(self):
+        ls = [key for key in self._data_vis.keys()]
+        ret = []
+        for nr, key in enumerate(ls):
+            ret.append({"criteria": key})
+
+        return ret
+
     def get_data_vis(self, key):
         """
         get code
@@ -183,6 +215,8 @@ class DynasaurDefinitions:
             d = self._abstat
         elif data_container == "abstat_cpm":
             d = self._abstat_cpm
+        elif data_container == "sleout":
+            d = self._sleout
 
         if name not in d.keys() or len(d) == 0:
             self._logger.emit(LOGConstants.ERROR[0],
@@ -397,6 +431,8 @@ class DynasaurDefinitions:
                 self._set_datacontainer_defintion(json_object, self._energy_global)
             elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.DISBOUT_PART:
                 self._set_datacontainer_defintion(json_object, self._part_disbout)
+            elif json_object[JsonConstants.TYPE] == ObjectConstantsForData.SLEOUT:
+                self._set_datacontainer_defintion(json_object, self._sleout)
             else:
                 self._logger.emit(LOGConstants.WARNING[0],
                                   json_object[JsonConstants.TYPE] + LoggerWARNING.print_statements[1])
@@ -445,7 +481,10 @@ class DynasaurDefinitions:
         """
         for definition in json_object[DefinitionConstants.DATA_VIS]:
             body_part = definition[JsonConstants.PART_OF]
-            self._data_vis[body_part + "_" + definition[JsonConstants.NAME]] = definition
+            if not body_part + "_" + definition[JsonConstants.NAME] in self._data_vis:
+                self._data_vis[body_part + "_" + definition[JsonConstants.NAME]] = definition
+            else:
+                self._logger.emit(LOGConstants.WARNING[0], "Double declaration of " + body_part + "_" + definition[JsonConstants.NAME])
 
     def _set_info(self):
         """
@@ -474,21 +513,66 @@ class DynasaurDefinitions:
 
     def read_def(self, fn):
         """
-        read definition file
+        read definition file(s)
 
         :param fn:
         :return:
         """
-        self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.READ % fn)
         if fn is None:
             exit("No definition file")
-        with open(fn) as fd:
-            try:
-                json_objects = json.load(fd)
-            except ValueError as e:
-                self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
-                exit("Exiting - Invalid JSON file")  # Because Mutant, if GUI "exit()" comment
-                return
+        if isinstance(fn, str):
+            self._logger.emit(LOGConstants.READ_DYNASAUR_DEF[0], LD.READ % fn)
+            with open(fn) as fd:
+                try:
+                    json_objects = json.load(fd)
+                except ValueError as e:
+                    self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
+                    exit("Exiting - Invalid JSON file")  # Because Mutant, if GUI "exit()" comment
+                    return
+        elif isinstance(fn, list):
+            if len(fn) < 1:
+                exit("No definition file(s) in list")
+            else:
+                first = True
+                for single_def_file in fn:
+                    with open(single_def_file) as fd:
+                        try:
+                            if first:
+                                json_objects = json.load(fd)
+                                first = False
+                                object_order = []
+                                for json_object in json_objects:
+                                    if DefinitionConstants.TITLE in json_object:
+                                        object_order.append(DefinitionConstants.TITLE)
+                                    elif DefinitionConstants.UNIT in json_object:
+                                        object_order.append(DefinitionConstants.UNIT)
+                                    elif DefinitionConstants.CRITERIA in json_object:
+                                        object_order.append(DefinitionConstants.CRITERIA)
+                                    elif DefinitionConstants.DATA_VIS in json_object:
+                                        object_order.append(DefinitionConstants.DATA_VIS)
+                                    elif DefinitionConstants.OBJECTS in json_object:
+                                        object_order.append(DefinitionConstants.OBJECTS)
+                            else:
+                                new_json_file = json.load(fd)
+                                for new_json_object in new_json_file:
+                                    key = list(new_json_object.keys())[0]
+                                    if key == DefinitionConstants.TITLE or key == DefinitionConstants.UNIT:
+                                        if json_objects[object_order.index(key)][key] != new_json_object[key]:
+                                            if key == DefinitionConstants.TITLE:
+                                                exit("Exiting - Different def-file titles!")
+                                            elif key == DefinitionConstants.UNIT:
+                                                exit("Exiting - Different def-file units!")
+                                    elif key == DefinitionConstants.CRITERIA or key == DefinitionConstants.DATA_VIS or key == DefinitionConstants.OBJECTS:
+                                        json_objects[object_order.index(key)][key] = json_objects[object_order.index(key)][key] + new_json_object[key]
+                                    else:
+                                        self._logger.emit(LOGConstants.ERROR[0], "json object not understood:" + str(json_object))
+                                        return
+                        except ValueError as e:
+                            self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[4] % e)
+                            exit("Exiting - Invalid JSON file")  # Because Mutant, if GUI "exit()" comment
+                            return
+        else:
+            exit("No definition file")
 
         # TestDefinitionJSON.test_def_json(json_objects, self._logger)
 
