@@ -4,6 +4,7 @@ from enum import Enum
 import numpy as np
 import time as t
 import copy
+import lasso
 
 
 from ..utils.constants import LOGConstants
@@ -1021,7 +1022,7 @@ class Glstat(BinoutData):
                           'num_bad_shells', 'energy_ratio', 'external_work', 'time_step', 'global_z_velocity',
                           'global_x_velocity', 'eroded_internal_energy', 'time', 'total_energy',
                           'system_damping_energy', 'hourglass_energy', 'sliding_interface_energy', 'cycle', 'ts_eltype',
-                          'joint_internal_energy', 'eroded_hourglass_energy', 'nzc', 'added_mass']
+                          'joint_internal_energy', 'eroded_hourglass_energy', 'nzc', 'added_mass','resultant_global_velocity','ts_part']
 
     ids_name = "ids"
 
@@ -1087,7 +1088,10 @@ class Matsum(BinoutData):
     DATA_CHANNEL_TYPES = ['x_momentum', 'y_rbvelocity', 'z_rbvelocity', 'internal_energy', 'kinetic_energy',
                           'max_shell_mass', 'y_momentum', 'mass', 'eroded_kinetic_energy', 'brick_id', 'max_brick_mass',
                           'shell_id', 'eroded_internal_energy', 'z_momentum', 'hourglass_energy',
-                          'eroded_hourglass_energy' 'x_rbvelocity']
+                          'eroded_hourglass_energy' 'x_rbvelocity','energy_ratio','eroded_energy_ratio','x_rbacceleration',
+                          'y_acceleration','z_acceleration','x_rbdisplacement','y_rbdisplacement','z_rbdisplacement',
+                          'resultant_displacement','resultant_velocity','resultant_acceleration','resultant_momentum',
+                          'x_acceleration','y_acceleration','z_acceleration']
 
     ids_name = "ids"
 
@@ -1111,7 +1115,9 @@ class Nodout(BinoutData):
                           'rz_displacement', 'rx_acceleration', 'ry_acceleration', 'rz_acceleration', 'x_coordinate',
                           'y_coordinate', 'z_coordinate', 'x_displacement', 'y_displacement', 'z_displacement',
                           'x_velocity', 'y_velocity', 'z_velocity', 'x_acceleration', 'y_acceleration',
-                          'z_acceleration']
+                          'z_acceleration','resultant_rotation','resultant_angular_velocity',
+                          'resultant_angular_acceleration','resultant_displacement','resultant_velocity',
+                          'resultant_acceleration']
     ids_name = "ids"
 
     def __init__(self, data, logger, dynasaur_definitions):
@@ -1207,7 +1213,7 @@ class JointForc(BinoutData):
 
 class RCForc(BinoutData):
     DATA_CHANNEL_TYPES = ['x_moment', 'y_moment', 'z_moment', 'x_force', 'y_force', 'z_force', 'tie_count', 'mass',
-                          'tie_area']
+                          'tie_area','resultant_force']
 
     ids_name = "ids"
 
@@ -1230,6 +1236,13 @@ class RCForc(BinoutData):
         :param channel_name:
         :return:
         """
+
+        if not isinstance(self._binout, lasso.dyna.Binout):
+            # call implementation
+            # used for madymo implementation
+            # maybe changed
+            return BinoutData.get_measurement_channel(self, id_, channel_name)
+
         if channel_name == 'time':
             return self._time.reshape(-1, 1)
 
@@ -1300,3 +1313,41 @@ class Secforc(BinoutData):
     def __get_indices_of_ids(self, ids):
         tmp_ids = self._ids.tolist()
         return [tmp_ids.index(val) for val in tmp_ids if val in ids]
+
+
+class Sleout(BinoutData):
+    """
+    """
+    DATA_CHANNEL_TYPES = ['total_friction', 'slave', 'total_master', 'master', 'cycle',
+                          'friction_energy', 'time', 'total_energy',
+                          'total_slave']
+    #TODO: Check TYPES with someone who is using this LS Dyna inputs
+    ids_name = "ids"
+
+    def __init__(self, binout, logger, dynasaur_definitions):
+        """
+        Initialization/constructor
+
+        :param: binout
+        :param: logger
+        :param: dynasaur definition
+
+        :return:
+        """
+        BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'sleout')
+
+    def get_measurement_channel(self, id_, channel_name):
+        """
+        returns the interpolated channel,
+        Aim: Constant time interval
+        :param id:
+        :param channel_name:
+        :return:
+        """
+        if channel_name == 'time':
+            return self._time.reshape(-1, 1)
+        d = self._data[channel_name][:, 0]
+        return d
+
+
+
