@@ -2,14 +2,15 @@ import h5py
 import numpy as np
 
 from ..utils.constants import LOGConstants, UnitsConstants, MadymoConstants, DataChannelTypesNodout, \
-    DataChannelTypesSecforc
+    DataChannelTypesSecforc, DataChannelTypesContact
 from ..utils.logger import ConsoleLogger
 from ..data.dynasaur_definitions import DynasaurDefinitions
+import re
 
 
 class MadymoData:
 
-    def _extratct_data(self, node, data_type, idx_madymo_data):
+    def _extratct_data(self, node, data_type, channel_type, idx_madymo_data):
         """
 
         :param node:
@@ -17,12 +18,17 @@ class MadymoData:
         :param idx_madymo_data:
         :return:
         """
-        for idx, id_ in enumerate(self._d['nodout']['ids']):
+        for idx, id_ in enumerate(self._d[data_type]['ids']):
             keys = [i for i in node.keys() if i.startswith("SIGNAL")]
             for key in keys:
-                id_to_compare = ''.join([chr(i) for i in list(node[key].attrs.items())[0][1] if i != 0]).split(" ")[0]
+                if data_type == "rcforc":
+                    id_string= ''.join([chr(i) for i in list(node[key].attrs.items())[0][1] if i != 0])
+                    id_to_compare = self._get_simplified_rcforc_id(id_string)
+                else:
+                    id_to_compare = ''.join([chr(i) for i in list(node[key].attrs.items())[0][1] if i != 0]).split(" ")[0]
+
                 if id_to_compare == id_:
-                    self._d["nodout"][data_type][:, idx] = node[key]["Y_VALUES"][idx_madymo_data, :]
+                    self._d[data_type][channel_type][:, idx] = node[key]["Y_VALUES"][idx_madymo_data, :]
 
     def _extract_data_energy(self, node, data_type, ids,  enegry_type):
         """
@@ -55,11 +61,6 @@ class MadymoData:
             elif id_string.startswith("work done by contact forces") and enegry_type == "sliding_interface_energy":
                 self._d[data_type]["sliding_interface_energy"][:, insert_idx] = node[key]["Y_VALUES"][0, :]
 
-
-
-            # print(''.join([chr(i) for i in list(node[key].attrs.items())[0][1] if i != 0]).strip().split(" ")[0])
-            # print(''.join([chr(i) for i in list(node[key].attrs.items())[0][1] if i != 0]).strip().split(" "))
-
     def _add_energy_ids(self, node, data_type):
         """
 
@@ -99,6 +100,25 @@ class MadymoData:
             if id_string.startswith("work done by contact forces"):
                 self._d[data_type]["sliding_interface_energy"] = None
 
+    def _get_simplified_rcforc_id(self, id_string):
+        id_string_ = id_string.split(" ")
+        xy = [re.search("(\/[0-9]+)+", i).group(0) for i in id_string_ if re.search("(\/[0-9]+)+", i) is not None]
+        if id_string.split(":")[-1].isnumeric():
+            # case
+            # /1000/101 ( /SystemInteractions_sys/Seat-Occupant_cntfrc ) contact.mb_fe /1000/54 ( /SystemInteractions_sys/ASM-Occupant_cnt ) :1
+            simiplified_id = "-".join(xy) + ":" + str(id_string.split(":")[-1])
+        elif id_string.split(":")[-1] == id_string:
+            # case
+            # /1000/999 ( /SystemInteractions_sys/Airbag-Occupant_cntfrc ) contact.fe_fe /1000/91 ( /SystemInteractions_sys/Occupant-Airbag_cnt ) CFC600
+            simiplified_id = "-".join(xy)
+        else:
+            # case
+            # /1000/1002 ( /SystemInteractions_sys/Airbag-Interior_cntfrc ) contact.mb_fe /1000/112 ( /SystemInteractions_sys/Seat-Airbag_cnt ) :1 CFC600
+            chn_nr = id_string.split(":")[-1].split("CFC")[0]
+            simiplified_id = "-".join(xy) + ":" + chn_nr
+
+        return simiplified_id
+
     def _add_ids(self, node, data_type):
         """
         add signal ids
@@ -110,11 +130,15 @@ class MadymoData:
         signals = [signal for signal in node.keys() if signal.startswith("SIGNAL")]
         for signal in signals:
             id_string = ''.join([chr(i) for i in list(node[signal].attrs.items())[0][1] if i != 0]).strip()
-            id_string = id_string.split(" ")[0]
-            assert (id_string not in self._d[data_type]["ids"])
+            if data_type == "rcforc":
+                simiplified_id = self._get_simplified_rcforc_id(id_string)
+            else:
+                simiplified_id = id_string.split(" ")[0]
+
+            assert (simiplified_id not in self._d[data_type]["ids"])
             # if assert: ID is already in id list
 
-            self._d[data_type]["ids"].append(id_string)
+            self._d[data_type]["ids"].append(simiplified_id)
 
     def _add_time(self, node, data_type):
         """
@@ -153,6 +177,24 @@ class MadymoData:
             #       mapping is done according to the identifiers in the output files i.e.
             f['MODEL_0'].visititems(self._get_available_data_types_dict)
 
+            if "rcforc" in self._d:
+                a = 0
+                b = 0
+                dimension = (self._d['rcforc']['time'].shape[0], len(self._d['rcforc']['ids']))
+                data_types_to_be_filled = [i for i in self._d['rcforc'].keys() if i != "ids" and i != "time"]
+                for data_type in data_types_to_be_filled:
+                    self._d['rcforc'][data_type] = np.empty(dimension)
+                    self._d['rcforc'][data_type][:] = np.nan
+
+                    if data_type == DataChannelTypesContact.X_FORCE or \
+                            data_type == DataChannelTypesContact.Y_FORCE or \
+                            data_type == DataChannelTypesContact.Z_FORCE:
+                        idx_madymo_data = 1 if data_type == DataChannelTypesContact.X_FORCE else (
+                            2 if data_type == DataChannelTypesContact.Y_FORCE else 3)
+
+                        self._extratct_data(f["MODEL_0/contact loads"], "rcforc", data_type,
+                                            idx_madymo_data=idx_madymo_data)
+
             # extend the data dictionary
             if "nodout" in self._d:
                 dimension = (self._d['nodout']['time'].shape[0], len(self._d['nodout']['ids']))
@@ -166,7 +208,7 @@ class MadymoData:
                             data_type == DataChannelTypesNodout.RZ_DISPLACEMENT:
                         idx_madymo_data = 0 if data_type == DataChannelTypesNodout.RX_DISPLACEMENT else (
                             1 if data_type == DataChannelTypesNodout.RY_DISPLACEMENT else 2)
-                        self._extratct_data(f["MODEL_0/Angular displacements"], data_type,
+                        self._extratct_data(f["MODEL_0/Angular displacements"], "nodout", data_type,
                                             idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.RX_VELOCITY or \
@@ -174,14 +216,15 @@ class MadymoData:
                             data_type == DataChannelTypesNodout.RZ_VELOCITY:
                         idx_madymo_data = 0 if data_type == DataChannelTypesNodout.RX_VELOCITY else (
                             1 if data_type == DataChannelTypesNodout.RY_VELOCITY else 2)
-                        self._extratct_data(f["MODEL_0/Angular velocities"], data_type, idx_madymo_data=idx_madymo_data)
+                        self._extratct_data(f["MODEL_0/Angular velocities"], "nodout",
+                                            data_type, idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.RX_ACCELERATION or \
                             data_type == DataChannelTypesNodout.RY_ACCELERATION or \
                             data_type == DataChannelTypesNodout.RZ_ACCELERATION:
                         idx_madymo_data = 0 if data_type == DataChannelTypesNodout.RX_ACCELERATION else (
                             1 if data_type == DataChannelTypesNodout.RY_ACCELERATION else 2)
-                        self._extratct_data(f["MODEL_0/Angular accelerations"], data_type,
+                        self._extratct_data(f["MODEL_0/Angular accelerations"], "nodout", data_type,
                                             idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.X_ACCELERATION or \
@@ -189,28 +232,32 @@ class MadymoData:
                             data_type == DataChannelTypesNodout.Z_ACCELERATION:
                         idx_madymo_data = 1 if data_type == DataChannelTypesNodout.X_ACCELERATION else (
                             2 if data_type == DataChannelTypesNodout.Y_ACCELERATION else 3)
-                        self._extratct_data(f["MODEL_0/Linear accelerations"], data_type, idx_madymo_data=idx_madymo_data)
+                        self._extratct_data(f["MODEL_0/Linear accelerations"],"nodout",
+                                            data_type, idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.X_VELOCITY or \
                             data_type == DataChannelTypesNodout.Y_VELOCITY or \
                             data_type == DataChannelTypesNodout.Z_VELOCITY:
                         idx_madymo_data = 1 if data_type == DataChannelTypesNodout.X_VELOCITY else (
                             2 if data_type == DataChannelTypesNodout.Y_VELOCITY else 3)
-                        self._extratct_data(f["MODEL_0/Linear velocities"], data_type, idx_madymo_data=idx_madymo_data)
+                        self._extratct_data(f["MODEL_0/Linear velocities"], "nodout",
+                                            data_type, idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.X_COORDINATE or \
                             data_type == DataChannelTypesNodout.Y_COORDINATE or \
                             data_type == DataChannelTypesNodout.Z_COORDINATE:
                         idx_madymo_data = 1 if data_type == DataChannelTypesNodout.X_COORDINATE else \
                             (2 if data_type == DataChannelTypesNodout.Y_COORDINATE else 3)
-                        self._extratct_data(f["MODEL_0/Linear positions"], data_type, idx_madymo_data=idx_madymo_data)
+                        self._extratct_data(f["MODEL_0/Linear positions"], "nodout", data_type,
+                                            idx_madymo_data=idx_madymo_data)
 
                     elif data_type == DataChannelTypesNodout.X_DISPLACEMENT or \
                             data_type == DataChannelTypesNodout.Y_DISPLACEMENT or \
                             data_type == DataChannelTypesNodout.Z_DISPLACEMENT:
                         idx_madymo_data = 1 if data_type == DataChannelTypesNodout.X_DISPLACEMENT else (
                             2 if data_type == DataChannelTypesNodout.Y_DISPLACEMENT else 3)
-                        self._extratct_data(f["MODEL_0/Linear displacements"], data_type, idx_madymo_data=idx_madymo_data)
+                        self._extratct_data(f["MODEL_0/Linear displacements"], "nodout", data_type,
+                                            idx_madymo_data=idx_madymo_data)
 
             if "matsum" in self._d:
                 dimension = (self._d['matsum']['time'].shape[0], len(self._d['matsum']['ids']))
@@ -296,6 +343,11 @@ class MadymoData:
                     if node.parent.name.split("/")[-1].startswith("Energy output"):
                         self._d['matsum'] = {"ids": [], "time": None}
 
+                if 'rcforc' not in self._d.keys():
+                    if node.parent.name.split("/")[-1].startswith("contact loads"):
+                        self._d['rcforc'] = {"ids": [], "time": None}
+
+
                 if node.parent.name.split("/")[-1].startswith("Energy output"):
                     # TODO
                     # self._d['glstat'][DataChannelTypesNodout.RX_ACCELERATION] = None
@@ -355,6 +407,13 @@ class MadymoData:
                     self._add_ids(node.parent, data_type="nodout")
                     self._add_time(node.parent, data_type="nodout")
 
+                elif node.parent.name.split("/")[-1].startswith("contact loads"):
+                    self._d['rcforc'][DataChannelTypesContact.X_FORCE] = None
+                    self._d['rcforc'][DataChannelTypesContact.Y_FORCE] = None
+                    self._d['rcforc'][DataChannelTypesContact.Z_FORCE] = None
+                    self._add_ids(node.parent, data_type="rcforc")
+                    self._add_time(node.parent, data_type="rcforc")
+
     def read(self, *argv):
         """
         read function to access Madymo data in a similar way as reading binout files in:
