@@ -29,11 +29,19 @@ class CriteriaController(Plugin, PluginInterface):
         self.init_plugin_data(update=True)
         self._data_dict = {}
 
-    def _calculate_and_store_results(self, param_dict):
+
+    def get_defined_calculation_procedures(self):
+        return self._dynasaur_definitions.get_criteria_calc_commands()
+
+    def _calculate_and_store_results(self, param_dict, to_si=False):
         """
         :param param_dict:
         :return:
         """
+
+        if to_si:
+            self._logger.emit(LOGConstants.WARNING[0], "Parameter to_si has no effect on criteria calculations!")
+
         self._logger.emit(LOGConstants.SCRIPT[0], "Calculating Criteria: " + param_dict["criteria"])
 
         sample_offset = self._get_sample_offset(param_dict)
