@@ -66,6 +66,7 @@ class Plugin(object):
         # ensure unique timestamps
         time.sleep(1)
 
+
     def _check_if_user_function(self, function_name):
         """
         :param function_name:
@@ -267,20 +268,28 @@ class Plugin(object):
 class PluginInterface(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
-    def _calculate_and_store_results(self, param_dict):
+    def _calculate_and_store_results(self, param_dict, to_si):
         """
         :param param_dict:
         :return:
         """
 
-    def calculate(self, param_dict):
+    @abc.abstractmethod
+    def get_defined_calculation_procedures(self):
+        """
+
+        :return:
+        """
+
+    def calculate(self, param_dict, to_si=False):
         """
 
         :param param_dict:
+        :param to_si
         :return:
         """
         param_dict_data = self._dynasaur_definitions.get_param_dict_from_command(command=param_dict)
-        self._calculate_and_store_results(param_dict_data)
+        self._calculate_and_store_results(param_dict_data, to_si=to_si)
 
     def get_object_data(self, type_, id_, strain_stress, index, channel, data_offset):
         """
